@@ -154,6 +154,8 @@ static PrefParam param[] = {
 	 &prefs_common.summary_col_visible[S_COL_SIZE], P_BOOL},
 	{"summary_col_show_number", "FALSE",
 	 &prefs_common.summary_col_visible[S_COL_NUMBER], P_BOOL},
+	{"summary_col_show_to", "FALSE",
+	 &prefs_common.summary_col_visible[S_COL_TO], P_BOOL},
 
 	{"summary_col_pos_mark", "0",
 	  &prefs_common.summary_col_pos[S_COL_MARK], P_INT},
@@ -171,6 +173,46 @@ static PrefParam param[] = {
 	  &prefs_common.summary_col_pos[S_COL_SIZE], P_INT},
 	{"summary_col_pos_number", "7",
 	  &prefs_common.summary_col_pos[S_COL_NUMBER], P_INT},
+	{"summary_col_pos_to", "8",
+	  &prefs_common.summary_col_pos[S_COL_TO], P_INT},
+
+	{"summary_sent_col_show_mark", "TRUE",
+	 &prefs_common.summary_sent_col_visible[S_COL_MARK], P_BOOL},
+	{"summary_sent_col_show_unread", "TRUE",
+	 &prefs_common.summary_sent_col_visible[S_COL_UNREAD], P_BOOL},
+	{"summary_sent_col_show_mime", "TRUE",
+	 &prefs_common.summary_sent_col_visible[S_COL_MIME], P_BOOL},
+	{"summary_sent_col_show_subject", "TRUE",
+	 &prefs_common.summary_sent_col_visible[S_COL_SUBJECT], P_BOOL},
+	{"summary_sent_col_show_from", "FALSE",
+	 &prefs_common.summary_sent_col_visible[S_COL_FROM], P_BOOL},
+	{"summary_sent_col_show_date", "TRUE",
+	 &prefs_common.summary_sent_col_visible[S_COL_DATE], P_BOOL},
+	{"summary_sent_col_show_size", "TRUE",
+	 &prefs_common.summary_sent_col_visible[S_COL_SIZE], P_BOOL},
+	{"summary_sent_col_show_number", "FALSE",
+	 &prefs_common.summary_sent_col_visible[S_COL_NUMBER], P_BOOL},
+	{"summary_sent_col_show_to", "TRUE",
+	 &prefs_common.summary_sent_col_visible[S_COL_TO], P_BOOL},
+
+	{"summary_sent_col_pos_mark", "0",
+	  &prefs_common.summary_sent_col_pos[S_COL_MARK], P_INT},
+	{"summary_sent_col_pos_unread", "1",
+	  &prefs_common.summary_sent_col_pos[S_COL_UNREAD], P_INT},
+	{"summary_sent_col_pos_mime", "2",
+	  &prefs_common.summary_sent_col_pos[S_COL_MIME], P_INT},
+	{"summary_sent_col_pos_subject", "3",
+	  &prefs_common.summary_sent_col_pos[S_COL_SUBJECT], P_INT},
+	{"summary_sent_col_pos_from", "8",
+	  &prefs_common.summary_sent_col_pos[S_COL_FROM], P_INT},
+	{"summary_sent_col_pos_date", "5",
+	  &prefs_common.summary_sent_col_pos[S_COL_DATE], P_INT},
+	{"summary_sent_col_pos_size", "6",
+	  &prefs_common.summary_sent_col_pos[S_COL_SIZE], P_INT},
+	{"summary_sent_col_pos_number", "7",
+	  &prefs_common.summary_sent_col_pos[S_COL_NUMBER], P_INT},
+	{"summary_sent_col_pos_to", "4",
+	  &prefs_common.summary_sent_col_pos[S_COL_TO], P_INT},
 
 	{"summary_col_size_mark", "10",
 	 &prefs_common.summary_col_size[S_COL_MARK], P_INT},
@@ -188,6 +230,8 @@ static PrefParam param[] = {
 	 &prefs_common.summary_col_size[S_COL_SIZE], P_INT},
 	{"summary_col_size_number", "40",
 	 &prefs_common.summary_col_size[S_COL_NUMBER], P_INT},
+	{"summary_col_size_to", "120",
+	 &prefs_common.summary_col_size[S_COL_TO], P_INT},
 
 	/* Widget size */
 	{"folderwin_x", "16", &prefs_common.folderwin_x, P_INT},
