@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -86,6 +86,8 @@ struct _Compose
 	GtkWidget *exteditor_btn;
 	GtkWidget *linewrap_btn;
 	GtkWidget *addrbook_btn;
+	GtkWidget *prefs_common_btn;
+	GtkWidget *prefs_account_btn;
 
 	GtkWidget *vbox2;
 
@@ -139,7 +141,8 @@ struct _Compose
 	ComposeMode mode;
 
 	MsgInfo *targetinfo;
-	MsgInfo *replyinfo;
+	gchar *reply_target;
+	gchar *forward_targets;
 
 	gchar	*replyto;
 	gchar	*cc;
