@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2002 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,9 +24,16 @@
 #include "defs.h"
 
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtkwindow.h>
+#include <gtk/gtkclist.h>
+#include <gtk/gtkvbox.h>
+#include <gtk/gtkstatusbar.h>
+#include <gtk/gtktable.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtkstock.h>
 
-#include "intl.h"
 #include "addressbook.h"
 #include "addressitem.h"
 #include "addrbook.h"
@@ -44,7 +51,7 @@
 #define ADDRESSBOOK_GUESS_FOLDER_NAME	"NewFolder"
 #define ADDRESSBOOK_GUESS_GROUP_NAME	"NewGroup"
 
-#define EDITGROUP_WIDTH      580
+#define EDITGROUP_WIDTH      600
 #define EDITGROUP_HEIGHT     340
 
 typedef enum {
@@ -59,6 +66,7 @@ typedef enum {
 
 static struct _GroupEdit_dlg {
 	GtkWidget *window;
+	GtkWidget *hbbox;
 	GtkWidget *ok_btn;
 	GtkWidget *cancel_btn;
 	GtkWidget *statusbar;
@@ -66,8 +74,8 @@ static struct _GroupEdit_dlg {
 
 	/* Basic data tab */
 	GtkWidget *entry_name;
-	GtkCList *clist_group;
 	GtkCList *clist_avail;
+	GtkCList *clist_group;
 
 	GHashTable *hashEMail;
 	gint rowIndGroup;
@@ -118,11 +126,12 @@ static gint edit_group_delete_event(GtkWidget *widget, GdkEventAny *event, gbool
 	return TRUE;
 }
 
-static void edit_group_key_pressed(GtkWidget *widget, GdkEventKey *event, gboolean *cancelled) {
+static gboolean edit_group_key_pressed(GtkWidget *widget, GdkEventKey *event, gboolean *cancelled) {
 	if (event && event->keyval == GDK_Escape) {
 		*cancelled = TRUE;
 		gtk_main_quit();
 	}
+	return FALSE;
 }
 
 static gchar *edit_group_format_item_clist( ItemPerson *person, ItemEMail *email ) {
@@ -158,11 +167,13 @@ static gint edit_group_clist_add_email( GtkCList *clist, ItemEMail *email ) {
 
 static void edit_group_load_clist( GtkCList *clist, GList *listEMail ) {
 	GList *node = listEMail;
+	gtk_clist_freeze( clist );
 	while( node ) {
 		ItemEMail *email = node->data;
 		edit_group_clist_add_email( clist, email );
 		node = g_list_next( node );
 	}
+	gtk_clist_thaw( clist );
 }
 
 static void edit_group_group_selected( GtkCList *clist, gint row, gint column, GdkEvent *event, gpointer data ) {
@@ -180,6 +191,7 @@ static gint edit_group_move_email( GtkCList *clist_from, GtkCList *clist_to, gin
 		gtk_clist_remove( clist_from, row );
 		rrow = edit_group_clist_add_email( clist_to, email );
 		gtk_clist_select_row( clist_to, rrow, 0 );
+		gtkut_clist_set_focus_row( clist_to, rrow );
 	}
 	return rrow;
 }
@@ -194,22 +206,24 @@ static void edit_group_to_avail( GtkWidget *widget, gpointer data ) {
 					groupeditdlg.clist_avail, groupeditdlg.rowIndGroup );
 }
 
-static void edit_group_list_group_button( GtkCList *clist, GdkEventButton *event, gpointer data ) {
-	if( ! event ) return;
+static gboolean edit_group_list_group_button( GtkCList *clist, GdkEventButton *event, gpointer data ) {
+	if( ! event ) return FALSE;
 	if( event->button == 1 ) {
 		if( event->type == GDK_2BUTTON_PRESS ) {
 			edit_group_to_avail( NULL, NULL );
 		}
 	}
+	return FALSE;
 }
 
-static void edit_group_list_avail_button( GtkCList *clist, GdkEventButton *event, gpointer data ) {
-	if( ! event ) return;
+static gboolean edit_group_list_avail_button( GtkCList *clist, GdkEventButton *event, gpointer data ) {
+	if( ! event ) return FALSE;
 	if( event->button == 1 ) {
 		if( event->type == GDK_2BUTTON_PRESS ) {
 			edit_group_to_group( NULL, NULL );
 		}
 	}
+	return FALSE;
 }
 
 static gint edit_group_list_compare_func( GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2 ) {
@@ -220,12 +234,13 @@ static gint edit_group_list_compare_func( GtkCList *clist, gconstpointer ptr1, g
 	if( cell2 ) name2 = cell2->u.text;
 	if( ! name1 ) return ( name2 != NULL );
 	if( ! name2 ) return -1;
-	return strcasecmp( name1, name2 );
+	return g_ascii_strcasecmp( name1, name2 );
 }
 
 static void addressbook_edit_group_create( gboolean *cancelled ) {
 	GtkWidget *window;
 	GtkWidget *vbox;
+	GtkWidget *vbox1;
 	GtkWidget *hbbox;
 	GtkWidget *ok_btn;
 	GtkWidget *cancel_btn;
@@ -239,13 +254,16 @@ static void addressbook_edit_group_create( gboolean *cancelled ) {
 	GtkWidget *hboxl;
 	GtkWidget *vboxl;
 	GtkWidget *hboxh;
+	GtkWidget *vboxb;
+	GtkWidget *vboxb1;
+	GtkWidget *hboxb;
 
 	GtkWidget *clist_swin;
-	GtkWidget *clist_group;
 	GtkWidget *clist_avail;
+	GtkWidget *clist_group;
 
-	GtkWidget *buttonGroup;
-	GtkWidget *buttonAvail;
+	GtkWidget *button_add;
+	GtkWidget *button_remove;
 	gint top;
 
 	gchar *titles[ GROUP_N_COLS ];
@@ -255,32 +273,33 @@ static void addressbook_edit_group_create( gboolean *cancelled ) {
 	titles[ GROUP_COL_EMAIL   ] = _("E-Mail Address");
 	titles[ GROUP_COL_REMARKS ] = _("Remarks");
 
-	window = gtk_window_new(GTK_WINDOW_DIALOG);
-	gtk_widget_set_usize(window, EDITGROUP_WIDTH, EDITGROUP_HEIGHT );
-	gtk_container_set_border_width(GTK_CONTAINER(window), 0);
+	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	gtk_widget_set_size_request(window, EDITGROUP_WIDTH, EDITGROUP_HEIGHT);
 	gtk_window_set_title(GTK_WINDOW(window), _("Edit Group Data"));
-	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
+	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER_ON_PARENT);
 	gtk_window_set_modal(GTK_WINDOW(window), TRUE);	
-	gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-			   GTK_SIGNAL_FUNC(edit_group_delete_event),
-			   cancelled);
-	gtk_signal_connect(GTK_OBJECT(window), "key_press_event",
-			   GTK_SIGNAL_FUNC(edit_group_key_pressed),
-			   cancelled);
+	gtk_widget_realize(window);
+	g_signal_connect(G_OBJECT(window), "delete_event",
+			 G_CALLBACK(edit_group_delete_event),
+			 cancelled);
+	g_signal_connect(G_OBJECT(window), "key_press_event",
+			 G_CALLBACK(edit_group_key_pressed),
+			 cancelled);
 
-	vbox = gtk_vbox_new( FALSE, 6 );
-	gtk_container_set_border_width(GTK_CONTAINER(vbox), BORDER_WIDTH);
-	gtk_widget_show( vbox );
+	vbox = gtk_vbox_new( FALSE, 4 );
 	gtk_container_add( GTK_CONTAINER( window ), vbox );
 
+	vbox1 = gtk_vbox_new( FALSE, 10 );
+	gtk_box_pack_start(GTK_BOX(vbox), vbox1, TRUE, TRUE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox1), 6);
+
 	/* Group area */
 	hboxg = gtk_hbox_new( FALSE, 0 );
-	gtk_box_pack_start(GTK_BOX(vbox), hboxg, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), hboxg, FALSE, FALSE, 0);
 
 	/* Data entry area */
-	table = gtk_table_new( 1, 3, FALSE);
+	table = gtk_table_new( 1, 2, FALSE);
 	gtk_box_pack_start(GTK_BOX(hboxg), table, TRUE, TRUE, 0);
-	gtk_container_set_border_width( GTK_CONTAINER(table), 4 );
 	gtk_table_set_row_spacings(GTK_TABLE(table), 0);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 4);
 
@@ -294,48 +313,15 @@ static void addressbook_edit_group_create( gboolean *cancelled ) {
 	gtk_table_attach(GTK_TABLE(table), entry_name, 1, 2, top, (top + 1), GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0, 0);
 
 	/* List area */
-	hboxl = gtk_hbox_new( FALSE, 6 );
-	gtk_container_set_border_width( GTK_CONTAINER(hboxl), 8 );
-	gtk_box_pack_start(GTK_BOX(vbox), hboxl, TRUE, TRUE, 0);
-
-	/* Group list */
-	vboxl = gtk_vbox_new( FALSE, 0 );
-	gtk_box_pack_start(GTK_BOX(hboxl), vboxl, TRUE, TRUE, 0);
-
-	hboxh = gtk_hbox_new( FALSE, 0 );
-	gtk_container_set_border_width( GTK_CONTAINER(hboxh), 4 );
-	gtk_box_pack_start(GTK_BOX(vboxl), hboxh, FALSE, FALSE, 0);
-	label = gtk_label_new(_("Addresses in Group"));
-	gtk_box_pack_start(GTK_BOX(hboxh), label, TRUE, TRUE, 0);
-	buttonAvail = gtk_button_new_with_label( _( " -> " ) );
-	gtk_box_pack_end(GTK_BOX(hboxh), buttonAvail, FALSE, FALSE, 0);
-
-	clist_swin = gtk_scrolled_window_new( NULL, NULL );
-	gtk_box_pack_start(GTK_BOX(vboxl), clist_swin, TRUE, TRUE, 0);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(clist_swin),
-				       GTK_POLICY_AUTOMATIC,
-				       GTK_POLICY_ALWAYS);
-
-	clist_group = gtk_clist_new_with_titles( GROUP_N_COLS, titles );
-	gtk_container_add( GTK_CONTAINER(clist_swin), clist_group );
-	gtk_clist_set_selection_mode( GTK_CLIST(clist_group), GTK_SELECTION_BROWSE );
-	gtk_clist_set_column_width( GTK_CLIST(clist_group), GROUP_COL_NAME, GROUP_COL_WIDTH_NAME );
-	gtk_clist_set_column_width( GTK_CLIST(clist_group), GROUP_COL_EMAIL, GROUP_COL_WIDTH_EMAIL );
-	gtk_clist_set_compare_func( GTK_CLIST(clist_group), edit_group_list_compare_func );
-	gtk_clist_set_auto_sort( GTK_CLIST(clist_group), TRUE );
-
-	for( i = 0; i < GROUP_N_COLS; i++ )
-		GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(clist_group)->column[i].button, GTK_CAN_FOCUS);
+	hboxl = gtk_hbox_new( FALSE, 8 );
+	gtk_box_pack_start(GTK_BOX(vbox1), hboxl, TRUE, TRUE, 0);
 
 	/* Available list */
-	vboxl = gtk_vbox_new( FALSE, 0 );
+	vboxl = gtk_vbox_new( FALSE, 4 );
 	gtk_box_pack_start(GTK_BOX(hboxl), vboxl, TRUE, TRUE, 0);
 
 	hboxh = gtk_hbox_new( FALSE, 0 );
-	gtk_container_set_border_width( GTK_CONTAINER(hboxh), 4 );
 	gtk_box_pack_start(GTK_BOX(vboxl), hboxh, FALSE, FALSE, 0);
-	buttonGroup = gtk_button_new_with_label( _( " <- " ) );
-	gtk_box_pack_start(GTK_BOX(hboxh), buttonGroup, FALSE, FALSE, 0);
 	label = gtk_label_new(_("Available Addresses"));
 	gtk_box_pack_end(GTK_BOX(hboxh), label, TRUE, TRUE, 0);
 
@@ -352,44 +338,90 @@ static void addressbook_edit_group_create( gboolean *cancelled ) {
 	gtk_clist_set_column_width( GTK_CLIST(clist_avail), GROUP_COL_EMAIL, GROUP_COL_WIDTH_EMAIL );
 	gtk_clist_set_compare_func( GTK_CLIST(clist_avail), edit_group_list_compare_func );
 	gtk_clist_set_auto_sort( GTK_CLIST(clist_avail), TRUE );
+	gtkut_clist_set_redraw( GTK_CLIST(clist_avail) );
 
 	for( i = 0; i < GROUP_N_COLS; i++ )
 		GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(clist_avail)->column[i].button, GTK_CAN_FOCUS);
 
-	/* Status line */
-	hsbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_end(GTK_BOX(vbox), hsbox, FALSE, FALSE, BORDER_WIDTH);
-	statusbar = gtk_statusbar_new();
-	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, BORDER_WIDTH);
+	/* Add/Remove button */
+	vboxb = gtk_vbox_new( FALSE, 0 );
+	gtk_box_pack_start(GTK_BOX(hboxl), vboxb, FALSE, FALSE, 0);
+
+	vboxb1 = gtk_vbox_new( FALSE, 8 );
+	gtk_box_pack_start(GTK_BOX(vboxb), vboxb1, TRUE, FALSE, 0);
+
+	button_add = gtk_button_new_with_label( _( "  ->  " ) );
+	gtk_box_pack_start(GTK_BOX(vboxb1), button_add, FALSE, FALSE, 0);
+
+	button_remove = gtk_button_new_with_label( _( "  <-  " ) );
+	gtk_box_pack_start(GTK_BOX(vboxb1), button_remove, FALSE, FALSE, 0);
+
+	/* Group list */
+	vboxl = gtk_vbox_new( FALSE, 4 );
+	gtk_box_pack_start(GTK_BOX(hboxl), vboxl, TRUE, TRUE, 0);
+
+	hboxh = gtk_hbox_new( FALSE, 0 );
+	gtk_box_pack_start(GTK_BOX(vboxl), hboxh, FALSE, FALSE, 0);
+	label = gtk_label_new(_("Addresses in Group"));
+	gtk_box_pack_start(GTK_BOX(hboxh), label, TRUE, TRUE, 0);
+
+	clist_swin = gtk_scrolled_window_new( NULL, NULL );
+	gtk_box_pack_start(GTK_BOX(vboxl), clist_swin, TRUE, TRUE, 0);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(clist_swin),
+				       GTK_POLICY_AUTOMATIC,
+				       GTK_POLICY_ALWAYS);
+
+	clist_group = gtk_clist_new_with_titles( GROUP_N_COLS, titles );
+	gtk_container_add( GTK_CONTAINER(clist_swin), clist_group );
+	gtk_clist_set_selection_mode( GTK_CLIST(clist_group), GTK_SELECTION_BROWSE );
+	gtk_clist_set_column_width( GTK_CLIST(clist_group), GROUP_COL_NAME, GROUP_COL_WIDTH_NAME );
+	gtk_clist_set_column_width( GTK_CLIST(clist_group), GROUP_COL_EMAIL, GROUP_COL_WIDTH_EMAIL );
+	gtk_clist_set_compare_func( GTK_CLIST(clist_group), edit_group_list_compare_func );
+	gtk_clist_set_auto_sort( GTK_CLIST(clist_group), TRUE );
+	gtkut_clist_set_redraw( GTK_CLIST(clist_group) );
+
+	for( i = 0; i < GROUP_N_COLS; i++ )
+		GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(clist_group)->column[i].button, GTK_CAN_FOCUS);
 
 	/* Button panel */
-	gtkut_button_set_create(&hbbox, &ok_btn, _("OK"),
-				&cancel_btn, _("Cancel"), NULL, NULL);
-	gtk_box_pack_end(GTK_BOX(vbox), hbbox, FALSE, FALSE, 0);
+	hboxb = gtk_hbox_new(FALSE, 8);
+	gtk_box_pack_start(GTK_BOX(vbox1), hboxb, FALSE, FALSE, 0);
+
+	gtkut_stock_button_set_create(&hbbox, &ok_btn, GTK_STOCK_OK,
+				      &cancel_btn, GTK_STOCK_CANCEL,
+				      NULL, NULL);
+	gtk_box_pack_end(GTK_BOX(hboxb), hbbox, FALSE, FALSE, 0);
 	gtk_widget_grab_default(ok_btn);
 
-	gtk_signal_connect(GTK_OBJECT(ok_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_group_ok), cancelled);
-	gtk_signal_connect(GTK_OBJECT(cancel_btn), "clicked",
-			   GTK_SIGNAL_FUNC(edit_group_cancel), cancelled);
+	g_signal_connect(G_OBJECT(ok_btn), "clicked",
+			 G_CALLBACK(edit_group_ok), cancelled);
+	g_signal_connect(G_OBJECT(cancel_btn), "clicked",
+			 G_CALLBACK(edit_group_cancel), cancelled);
+
+	/* Status line */
+	hsbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_end(GTK_BOX(vbox), hsbox, FALSE, FALSE, 0);
+	statusbar = gtk_statusbar_new();
+	gtk_box_pack_start(GTK_BOX(hsbox), statusbar, TRUE, TRUE, 0);
 
 	gtk_widget_show_all(vbox);
 
 	/* Event handlers */
-	gtk_signal_connect( GTK_OBJECT(clist_group), "select_row",
-			GTK_SIGNAL_FUNC( edit_group_group_selected), NULL );
-	gtk_signal_connect( GTK_OBJECT(clist_avail), "select_row",
-			GTK_SIGNAL_FUNC( edit_group_avail_selected), NULL );
-	gtk_signal_connect( GTK_OBJECT(buttonGroup), "clicked",
-			GTK_SIGNAL_FUNC( edit_group_to_group ), NULL );
-	gtk_signal_connect( GTK_OBJECT(buttonAvail), "clicked",
-			GTK_SIGNAL_FUNC( edit_group_to_avail ), NULL );
-	gtk_signal_connect(GTK_OBJECT(clist_avail), "button_press_event",
-			   GTK_SIGNAL_FUNC(edit_group_list_avail_button), NULL);
-	gtk_signal_connect(GTK_OBJECT(clist_group), "button_press_event",
-			   GTK_SIGNAL_FUNC(edit_group_list_group_button), NULL);
+	g_signal_connect(G_OBJECT(clist_group), "select_row",
+			 G_CALLBACK( edit_group_group_selected), NULL);
+	g_signal_connect(G_OBJECT(clist_avail), "select_row",
+			 G_CALLBACK( edit_group_avail_selected), NULL);
+	g_signal_connect(G_OBJECT(button_add), "clicked",
+			 G_CALLBACK( edit_group_to_group ), NULL);
+	g_signal_connect(G_OBJECT(button_remove), "clicked",
+			 G_CALLBACK( edit_group_to_avail ), NULL);
+	g_signal_connect(G_OBJECT(clist_avail), "button_press_event",
+			 G_CALLBACK(edit_group_list_avail_button), NULL);
+	g_signal_connect(G_OBJECT(clist_group), "button_press_event",
+			 G_CALLBACK(edit_group_list_group_button), NULL);
 
 	groupeditdlg.window     = window;
+	groupeditdlg.hbbox      = hbbox;
 	groupeditdlg.ok_btn     = ok_btn;
 	groupeditdlg.cancel_btn = cancel_btn;
 	groupeditdlg.statusbar  = statusbar;
@@ -434,10 +466,12 @@ ItemGroup *addressbook_edit_group( AddressBookFile *abf, ItemFolder *parent, Ite
 
 	if (!groupeditdlg.window)
 		addressbook_edit_group_create(&cancelled);
+	gtkut_box_set_reverse_order(GTK_BOX(groupeditdlg.hbbox),
+				    !prefs_common.comply_gnome_hig);
 	gtk_widget_grab_focus(groupeditdlg.ok_btn);
 	gtk_widget_grab_focus(groupeditdlg.entry_name);
-	gtk_widget_show(groupeditdlg.window);
 	manage_window_set_transient(GTK_WINDOW(groupeditdlg.window));
+	gtk_widget_show(groupeditdlg.window);
 
 	/* Clear all fields */
 	groupeditdlg.rowIndGroup = -1;
@@ -460,9 +494,12 @@ ItemGroup *addressbook_edit_group( AddressBookFile *abf, ItemFolder *parent, Ite
 	listEMail = addrbook_get_available_email_list( abf, group );
 	edit_group_load_clist( groupeditdlg.clist_avail, listEMail );
 	mgu_clear_list( listEMail );
+	g_list_free( listEMail );
 	listEMail = NULL;
 	gtk_clist_select_row( groupeditdlg.clist_group, 0, 0 );
+	gtkut_clist_set_focus_row( groupeditdlg.clist_group, 0 );
 	gtk_clist_select_row( groupeditdlg.clist_avail, 0, 0 );
+	gtkut_clist_set_focus_row( groupeditdlg.clist_avail, 0 );
 
 	edit_group_status_show( _edit_group_dfl_message_ );
 
@@ -517,7 +554,7 @@ ItemFolder *addressbook_edit_folder( AddressBookFile *abf, ItemFolder *parent, I
 		return NULL;
 	}
 	if( folder ) {
-		if( strcasecmp( name, ADDRITEM_NAME(folder) ) == 0 ) {
+		if( g_ascii_strcasecmp( name, ADDRITEM_NAME(folder) ) == 0 ) {
 			g_free( name );
 			return NULL;
 		}
