@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtkversion.h>
 #include <gtk/gtkfilechooserdialog.h>
 #include <gtk/gtkexpander.h>
 #include <gtk/gtkstock.h>
@@ -30,6 +31,7 @@
 #include "alertpanel.h"
 #include "utils.h"
 #include "prefs_common.h"
+#include "inc.h"
 
 static GSList *filesel_select_file_full	(const gchar		*title,
 					 const gchar		*file,
@@ -43,6 +45,12 @@ static void filesel_save_expander_set_expanded	   (GtkWidget	*dialog,
 						    gboolean	 expanded);
 static gboolean filesel_save_expander_get_expanded (GtkWidget	*dialog);
 
+#if GTK_CHECK_VERSION(2, 8, 0)
+static GtkFileChooserConfirmation filesel_confirm_overwrite_cb
+					(GtkFileChooser		*chooser,
+					 gpointer		 data);
+#endif
+
 gchar *filesel_select_file(const gchar *title, const gchar *file,
 			   GtkFileChooserAction action)
 {
@@ -103,11 +111,23 @@ static GSList *filesel_select_file_full(const gchar *title, const gchar *file,
 			(dialog, save_expander_expanded);
 	}
 
+#if GTK_CHECK_VERSION(2, 8, 0)
+	if (action == GTK_FILE_CHOOSER_ACTION_SAVE) {
+		gtk_file_chooser_set_do_overwrite_confirmation
+			(GTK_FILE_CHOOSER(dialog), TRUE);
+		g_signal_connect(GTK_FILE_CHOOSER(dialog), "confirm-overwrite",
+				 G_CALLBACK(filesel_confirm_overwrite_cb),
+				 NULL);
+	}
+#endif
+
 	gtk_widget_show(dialog);
 
 	change_dir(prev_dir);
 	g_free(prev_dir);
 
+	inc_lock();
+
 	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
 		list = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(dialog));
 		if (list) {
@@ -119,6 +139,8 @@ static GSList *filesel_select_file_full(const gchar *title, const gchar *file,
 		}
 	}
 
+	inc_unlock();
+
 	if (action == GTK_FILE_CHOOSER_ACTION_SAVE)
 		save_expander_expanded =
 			filesel_save_expander_get_expanded(dialog);
@@ -136,17 +158,19 @@ gchar *filesel_save_as(const gchar *file)
 	filename = filesel_select_file(_("Save as"), file,
 				       GTK_FILE_CHOOSER_ACTION_SAVE);
 
+#if !GTK_CHECK_VERSION(2, 8, 0)
 	if (filename && is_file_exist(filename)) {
 		AlertValue aval;
 
-		aval = alertpanel(_("Overwrite"),
-				  _("Overwrite existing file?"),
-				  GTK_STOCK_OK, GTK_STOCK_CANCEL, NULL);
+		aval = alertpanel(_("Overwrite existing file"),
+				  _("The file already exists. Do you want to replace it?"),
+				  GTK_STOCK_YES, GTK_STOCK_NO, NULL);
 		if (G_ALERTDEFAULT != aval) {
 			g_free(filename);
 			filename = NULL;
 		}
 	}
+#endif
 
 	return filename;
 }
@@ -237,3 +261,32 @@ static gboolean filesel_save_expander_get_expanded(GtkWidget *dialog)
 	else
 		return FALSE;
 }
+
+#if GTK_CHECK_VERSION(2, 8, 0)
+static GtkFileChooserConfirmation filesel_confirm_overwrite_cb
+					(GtkFileChooser		*chooser,
+					 gpointer		 data)
+{
+	gchar *filename;
+	GtkFileChooserConfirmation ret =
+		GTK_FILE_CHOOSER_CONFIRMATION_ACCEPT_FILENAME;
+
+	filename = gtk_file_chooser_get_filename(chooser);
+
+	if (filename && is_file_exist(filename)) {
+		AlertValue aval;
+
+		aval = alertpanel(_("Overwrite existing file"),
+				  _("The file already exists. Do you want to replace it?"),
+				  GTK_STOCK_YES, GTK_STOCK_NO, NULL);
+		if (G_ALERTDEFAULT == aval)
+			ret = GTK_FILE_CHOOSER_CONFIRMATION_ACCEPT_FILENAME;
+		else
+			ret = GTK_FILE_CHOOSER_CONFIRMATION_SELECT_AGAIN;
+	}
+
+	g_free(filename);
+
+	return ret;
+}
+#endif
